@@ -26,7 +26,7 @@ use Tk::Pod::Util qw(is_in_path is_interactive detect_window_manager start_brows
 use vars qw($VERSION @ISA @POD $IDX
 	    @tempfiles @gv_pids $terminal_fallback_warn_shown);
 
-$VERSION = '5.29';
+$VERSION = '5.33';
 
 @ISA = qw(Tk::Frame Tk::Pod::SimpleBridge Tk::Pod::Cache);
 
@@ -45,23 +45,45 @@ BEGIN {
   DEBUG and warn "POD: @POD\n";
 };
 
-use Class::Struct;
-struct '_HistoryEntry' => [
-    'file'  => '$',
-    'text'  => '$',
-    'index' => '$',
-];
-sub _HistoryEntry::create {
-    my $o = shift->new;
-    my($what, $index) = @_;
-    if (ref $what eq 'HASH') {
-	$o->file($what->{file});
-	$o->text($what->{text});
-    } else {
-	$o->file($what);
+{
+    package # hide from CPAN indexer
+	Tk::Pod::Text::_HistoryEntry;
+
+    use File::Basename qw(basename);
+
+    for my $member (qw(file text index pod_title)) {
+	my $sub = sub {
+	    my $self = shift;
+	    if (@_) {
+		$self->{$member} = $_[0];
+	    }
+	    $self->{$member};
+	};
+	no strict 'refs';
+	*{$member} = $sub;
+    }
+
+    sub create {
+	my($class,$what,$index) = @_;
+	my $o = bless {}, $class;
+	if (ref $what eq 'HASH') {
+	    $o->file($what->{file});
+	    $o->text($what->{text});
+	} else {
+	    $o->file($what);
+	}
+	$o->index($index);
+	$o;
+    }
+
+    sub get_label {
+	my $self = shift;
+	my $pod_title = $self->pod_title;
+	return $pod_title if defined $pod_title;
+	my $file = $self->file;
+	return basename $file if defined $file;
+	return "<Untitled document>";
     }
-    $o->index($index);
-    $o;
 }
 
 use constant HISTORY_DIALOG_ARGS => [-icon => 'info',
@@ -149,10 +171,11 @@ sub file {   # main entry point
       my $file = shift;
       $w->_remember_old;
       eval {
+	  my $calling_from_history = $w->privateData()->{'from_history'};
 	  $w->{'File'} = $file;
 	  $w->{'Text'} = undef;
 	  my $path = $w->findpod($file);
-	  if (!$w->privateData()->{'from_history'}) {
+	  if (!$calling_from_history) {
 	      $w->history_modify_entry;
 	      $w->history_add({file => $path}, "1.0");
 	  }
@@ -170,6 +193,9 @@ sub file {   # main entry point
 	  if (!$w->get_from_cache) {
 	      $w->process($path);
 	      $w->add_to_cache; # XXX pass time for processing?
+	      if (!$calling_from_history) {
+		  $w->history_modify_current_title; # now the pod_title is known
+	      }
 	  }
 	  if (defined $t) {
 	      print Benchmark::timediff(Benchmark->new, $t)->timestr,"\n";
@@ -191,9 +217,10 @@ sub text {
       my $text = shift;
       $w->_remember_old;
       eval {
+	  my $calling_from_history = $w->privateData()->{'from_history'};
 	  $w->{'Text'} = $text;
 	  $w->{'File'} = undef;
-	  if (!$w->privateData()->{'from_history'}) {
+	  if (!$calling_from_history) {
 	      $w->history_modify_entry;
 	      $w->history_add({text => $text}, "1.0");
 	  }
@@ -213,6 +240,9 @@ sub text {
 	  # XXX title: the 2nd part of the hack
 	  my $title = $w->cget(-title);
 	  $w->process(\$text, $title);
+	  if (!$calling_from_history) {
+	      $w->history_modify_current_title; # now the pod_title is known
+	  }
 	  if (defined $t) {
 	      print Benchmark::timediff(Benchmark->new, $t)->timestr,"\n";
 	  }
@@ -250,7 +280,7 @@ sub _get_editable_path
    my $text = $w->cget("-text");
    $w->_need_File_Temp;
    my($fh,$fname) = File::Temp::tempfile(UNLINK => 1,
-					 SUFFIX => ".pod");
+					 SUFFIX => "_tkpod.pod");
    print $fh $text;
    close $fh;
    $path = $fname;
@@ -347,11 +377,23 @@ sub edit_get_linenumber
 sub get_linenumber
 {
  my($w) = @_;
- for my $tag ($w->tagNames('@' . ($w->{MenuX} - $w->rootx) . ',' . ($w->{MenuY} - $w->rooty)))
+ my $coord;
+ if (defined $w->{MenuX}) # called from a popup menu
   {
-   if ($tag =~ m{start_line_(\d+)})
+   $coord = '@' . ($w->{MenuX} - $w->rootx) . ',' . ($w->{MenuY} - $w->rooty);
+  }
+ else # called from a keyboard shortcut
+  {
+   $coord = '@0,0';
+  }
+ for my $try_line (0 .. 3) # top line resp. line under cursor might be empty, without tags, so try also the following ones
+  {
+   for my $tag ($w->tagNames($coord . ($try_line ? " + $try_line lines" : '')))
     {
-     return $1;
+     if ($tag =~ m{start_line_(\d+)})
+      {
+       return $1;
+      }
     }
   }
  undef;
@@ -734,22 +776,8 @@ sub Link_url {
     my ($w,$how,$index,$man,$sec) = @_;
     if (my($lat,$lon) = $man =~ m{^geo:([^,]+),([^,]+)}) {
         DEBUG and warn "Translate geo URI $man\n";
-        # XXX currently hardcoded to Wikipedia's GeoHack page, maybe make
-        # configurable
-        my $ddd2dms = sub {
-            my($ddd) = @_;
-	    my $north_east = $ddd >= 0;
-	    $ddd = -$ddd if !$north_east;
-	    my $deg = int($ddd);
-	    my $min = ($ddd-$deg)*60;
-	    my $sec = ($min-int($min))*60;
-	    $min = int($min);
-	    (($north_east ? $deg : -$deg), $min, $sec);
-	};
-	my $lat_sgn = $lat < 0 ? do { $lat *= -1; "S" } : "N";
-	my $lon_sgn = $lon < 0 ? do { $lon *= -1; "W" } : "E";
-        $man = "http://toolserver.org/~geohack/geohack.php?params="
-	    . join("_", $ddd2dms->($lat), $lat_sgn, $ddd2dms->($lon), $lon_sgn);
+        # XXX currently hardcoded to OSM, maybe make configurable
+        $man = "http://www.openstreetmap.org/?mlat=$lat&mlon=$lon";
     }
     DEBUG and warn "Start browser with $man\n";
     start_browser($man);
@@ -905,6 +933,17 @@ sub SearchFullText {
 	# XXX A very rough solution:
 	$IDX->Button(-text => "Rebuild search index",
 		     -command => sub {
+		      my $installscriptdir = $Config{'installscript'};
+		      my $perlindex = 'perlindex';
+		      if (-d $installscriptdir)
+		       {
+			$perlindex = "$installscriptdir/perlindex";
+			if (!-f $perlindex)
+			 {
+			  $w->_error_dialog("perlindex was expected in the path '$perlindex', but not found. Cannot build search index.");
+			  return;
+			 }
+		       }
 		      my $pw_bg_msg = "The next dialog will ask for the root password. The search index building will happen in background.";
 		      if (!is_in_path("gksu"))
 		       {
@@ -917,7 +956,7 @@ sub SearchFullText {
 			if (fork == 0)
 		         {
 			  system('xsu',
-				 '--command', 'perlindex -index',
+				 '--command', "$perlindex -index",
 				 '--username', 'root',
 				 '--title' => 'Rebuild search index',
 				 '--set-display' => $w->screen,
@@ -933,7 +972,7 @@ sub SearchFullText {
 			  system('gksu',
 				 '--user', 'root',
 				 #'--description', 'Rebuild search index',
-				 'perlindex -index',
+				 "perlindex -index",
 				);
 			  CORE::exit(0);
 			 }
@@ -970,7 +1009,7 @@ sub Print {
 	$text = $w->cget("-text");
 	$w->_need_File_Temp;
 	my($fh,$fname) = File::Temp::tempfile(UNLINK => 1,
-					      SUFFIX => ".pod");
+					      SUFFIX => "_tkpod.pod");
 	print $fh $text;
 	close $fh;
 	$path = $fname;
@@ -1013,7 +1052,7 @@ sub _print_pod_unix {
 	if ($gv) {
 	    $w->_need_File_Temp;
 
-	    my($fh,$fname) = File::Temp::tempfile(SUFFIX => ".ps");
+	    my($fh,$fname) = File::Temp::tempfile(SUFFIX => "_tkpod.ps");
 	    system("$pod2ps_pipe > $fname");
 	    push @tempfiles, $fname;
 	    my $pid = fork;
@@ -1125,7 +1164,7 @@ sub history_add {
 	}
     }
     my $hist = $w->privateData()->{history};
-    my $hist_entry = _HistoryEntry->create($what, $index);
+    my $hist_entry = Tk::Pod::Text::_HistoryEntry->create($what, $index, $w->{pod_title});
     $hist->[++$w->privateData()->{history_index}] = $hist_entry;
     splice @$hist, $w->privateData()->{history_index}+1;
     $w->history_view_update;
@@ -1241,10 +1280,23 @@ sub history_set {
 sub history_modify_entry {
     my $w = shift;
     if ($w->privateData()->{'history_index'} >= 0) {
-	my $old_entry = _HistoryEntry->create({file => $w->cget('-path'),
-					       text => $w->cget('-text')
-					      }, $w->index('@0,0'));
-	$w->privateData()->{'history'}->[$w->privateData()->{'history_index'}] = $old_entry;
+	my $entry = $w->privateData()->{'history'}->[$w->privateData()->{'history_index'}];
+	$entry->index($w->index('@0,0'));
+    }
+}
+
+# Modify the pod title of the current history entry.
+sub history_modify_current_title {
+    my $w = shift;
+    my $pod_title = $w->{pod_title};
+    if (defined $pod_title) {
+	my $history_index = $w->privateData()->{'history_index'};
+	if ($history_index >= 0) {
+	    my $entry = $w->privateData()->{'history'}->[$history_index];
+	    $entry->pod_title($pod_title);
+	    $w->history_view_update;
+	    $w->history_view_select;
+	}
     }
 }
 
@@ -1287,8 +1339,7 @@ sub history_view_update {
 	my $lb = $t->Subwidget('Lb');
 	$lb->delete(0, "end");
 	foreach my $histentry (@{$w->privateData()->{'history'}}) {
-	    (my $basename = $histentry->file) =~ s|^.*/([^/]+)$|$1|;
-	    $lb->insert("end", $basename);
+	    $lb->insert("end", $histentry->get_label);
 	}
     }
 }
@@ -1346,8 +1397,15 @@ sub _warn_dialog {
     );
 }
 
-END {
+sub cleanup_tempfiles {
     if (@tempfiles) {
+	# first get rid of all possible zombies
+	# before we can check with kill 0 => ...
+	require POSIX;
+	if (defined &POSIX::WNOHANG) { # defined everywhere?
+	    while (waitpid(-1, &POSIX::WNOHANG) > 0) { }
+	}
+
 	my $gv_running;
 	for my $pid (@gv_pids) {
 	    if (kill 0 => $pid) {
@@ -1362,10 +1420,15 @@ END {
 	    for my $temp (@tempfiles) {
 		unlink $temp;
 	    }
+	    @tempfiles = ();
 	}
     }
 }
 
+END {
+    cleanup_tempfiles();
+}
+
 1;
 
 __END__
@@ -1532,7 +1595,7 @@ Pod with umlaut: L<ExtUtils::MakeMaker>.
 
 Details:  L<perlpod> or perl, perlfunc.
 
-External links: L<http://www.cpan.org> (URL), L<< URL with link text|http://www.cpan.org >>, L<perl(1)> (man page).
+External links: L<http://www.cpan.org> (URL), L<< URL with link text|http://www.cpan.org >>, L<perl(1)> (man page), L<Berliner Fernsehturm|geo:52.520685,13.409461> (geo: URL)
 
 Links to local sections: L<a section (SYNOPSIS)|/SYNOPSIS>, L<an item
 (-file, currently wrong)|/-file>, L<a working item (auml)|/auml>.
@@ -1619,13 +1682,14 @@ Nick Ing-Simmons <F<nick@ni-s.u-net.com>>
 
 Current maintainer is Slaven ReziE<0x107> <F<slaven@rezic.de>>.
 
-Copyright (c) 1998 Nick Ing-Simmons.  All rights reserved.  This program
-is free software; you can redistribute it and/or modify it under the same
-terms as Perl itself.
+Copyright (c) 1998 Nick Ing-Simmons.
+Copyright (c) 2015,2016 Slaven Rezic.
+All rights reserved. This program is free software; you can
+redistribute it and/or modify it under the same terms as Perl itself.
 
 =cut
 
 # Local Variables:
 # mode: cperl
-# cperl-indent-level: 1
+# cperl-indent-level: 4
 # End:
